'use strict';

define(['underscore', 'jquery', 'routing', 'pim/form/common/save', 'pim/template/form/save'], function (
  _,
  $,
  Routing,
  SaveForm,
  template
) {
  return SaveForm.extend({
    template: _.template(template),
    events: {
      'click .save': 'save',
    },

    /**
     * {@inheritdoc}
     */
    render: function () {
      this.$el.html(
        this.template({
          label: _.__('pim_common.save'),
        })
      );
    },

    /**
     * {@inheritdoc}
     */
    save: function () {
      this.getRoot().trigger('pim_enrich:form:entity:pre_save', this.getFormData());
      this.showLoadingMask();

      $.ajax({
        method: 'POST',
        url: this.getSaveUrl(),
        contentType: 'application/json',
        data: JSON.stringify(this.getFormData()),
      })
        .then(this.postSave.bind(this))
        .fail(this.fail.bind(this))
        .always(this.hideLoadingMask.bind(this));
    },

    /**
     * {@inheritdoc}
     */
    getSaveUrl: function () {
      return Routing.generate(__moduleConfig.route);
    },

    /**
     * {@inheritdoc}
     */
    postSave: function (data) {
      this.setData(data);
      this.getRoot().trigger('pim_enrich:form:entity:post_fetch', data);

      SaveForm.prototype.postSave.apply(this, arguments);
    },
  });
});
